/**************************************************************************
 *                                                                        *
 *   Copyright (c) Digital Equipment  Corporation,  1991.   All  Rights   *
 *   Reserved.  Unpublished rights reserved under the copyright laws of   *
 *   the United States.                                                   *
 *                                                                        *
 *   The software  contained  on  this  media  is  proprietary  to  and   *
 *   embodies   the   confidential   technology  of  Digital  Equipment   *
 *   Corporation.  Possession, use, duplication or dissemination of the   *
 *   software  and media is authorized only pursuant to a valid written   *
 *   license from Digital Equipment Corporation.                          *
 *                                                                        *
 *   RESTRICTED RIGHTS LEGEND Use, duplication, or  disclosure  by  the   *
 *   U.S.   Government  is  subject  to  restrictions  as  set forth in   *
 *   Subparagraph  (c)(1)(ii)  of  DFARS  252.227-7013,   or   in   FAR   *
 *   52.227-19, as applicable.                                            *
 *                                                                        *
 **************************************************************************/
#ifdef SCCS
static char sccsid[]="@(#)read_vol.c	1.4 Stardent 90/03/02";
#endif
/*
			Copyright (c) 1989 by
			Stardent Computer Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Stardent Computer Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
	
	This file is under sccs control at Stardent in:
	/sccs3.p/avs/examples/s.read_vol.c
	
*/
/******************************************************************************
 *
 * read_volume  - read a volume from a file:
 *         width  (1 byte char)                     
 *         height (1 byte char)                     
 *         depth  (1 byte char)                     
 *         data   (width * height * depth * 1_byte_char)                     
 *
 * NOTE: The format of this file is unnecessarily restrictive due to the 
 *       simplicity of the header.  The format will be upgraded in the near 
 *       future to accomodate additional information and data types.
 *
 ******************************************************************************/

#include <stdio.h>
#include <avs/avs.h>
#include <avs/field.h>
#include <avs/flow.h>

/* Used for AVSmessage() */
static char file_version[] = "@(#)read_vol.c	1.4 Stellar 90/03/02";

/******************************************************************************/
AVSinit_modules()
{
    int read_volume();

    AVSmodule_from_desc(read_volume);
}

/******************************************************************************/
 static
read_volume()
{
    int param;
    int read_volume_compute();
    
    /* Set module name */
    AVSset_module_name("read volume c", MODULE_DATA);
    
    /* Create output port */
    AVScreate_output_port("Field Output", "field 3D scalar byte");
    
    /* Create parameter */
    param = AVSadd_parameter("Read Volume Browser", "string", 0, 0, ".dat");
    AVSconnect_widget(param, "browser");
    
    /* Set compute procedure */
    AVSset_compute_proc(read_volume_compute);
}
	
/******************************************************************************/
 static
read_volume_compute(data, filename)
    AVSfield_char **data;
    char *filename;
{
    AVSfield_char *input;
    FILE *pf;
    int dims[3];
    unsigned char cx, cy, cz;
    
    /* No data file name yet (not an error) */
    if (!filename)
      return(1);
    
    /* Attempt to open file */
    if (!(pf = fopen(filename, "r"))) {
	AVSmessage(file_version, AVS_Warning, AVSmodule, "read_volume_compute",
		   "Ok", "Can't open data file %s", filename);
	return(0);
    }
    
    /* Free old memory */
    if (*data) 
      AVSfield_free(*data);
    
    /* Read the dimensions */
    if (fread(&cx, sizeof(unsigned char), 1, pf) != 1) {
	AVSmessage(file_version, AVS_Warning, AVSmodule, "read_volume_compute",
		   "Ok", "Error reading header from %s", filename);
	fclose(pf);
	return(0);
    }
    if (fread(&cy, sizeof(unsigned char), 1, pf) != 1) {
	AVSmessage(file_version, AVS_Warning, AVSmodule, "read_volume_compute",
		   "Ok", "Error reading header from %s", filename);
	fclose(pf);
	return(0);
    }
    if (fread(&cz, sizeof(unsigned char), 1, pf) != 1) {
	AVSmessage(file_version, AVS_Warning, AVSmodule, "read_volume_compute",
		   "Ok", "Error reading header from %s", filename);
	fclose(pf);
	return(0);
    }

    /* Set the dimensions */
    dims[0] = cx;
    dims[1] = cy;
    dims[2] = cz;
    
    /* Allocate space for volume data */
    input = (AVSfield_char*)AVSdata_alloc("field 3D scalar byte uniform", dims);

    /* Read the data */
    if (fread(input->data, sizeof(char), (int)(cx*cy*cz), pf) != (int) (cx * cy * cz)) {
	AVSmessage(file_version, AVS_Warning, AVSmodule, "read_volume_compute",
		   "Ok", "Error reading data from %s", filename);
	AVSfield_free(input);
	fclose(pf);
	return(0);
    }
    
    /* Set the input pointer */
    *data = input;
    
    /* Close the file */
    fclose(pf);

    /* Indicate success */
    return(1);
}
